// 
// The LibSea Graph Library.
// Copyright (C) 2000,2001,2002 The Regents of the University of California.
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
// 
// ######END_HEADER######
// 


package org.caida.libsea;

public class Float3Array extends HeapArray
{
    ////////////////////////////////////////////////////////////////////////
    // CONSTRUCTORS
    ////////////////////////////////////////////////////////////////////////

    public Float3Array()
    {
	super(1);
    }

    // {segmentSize} must be a power of 2.
    public Float3Array(int segmentSize)
    {
	super(1, segmentSize);
    }

    ////////////////////////////////////////////////////////////////////////
    // PUBLIC METHODS
    ////////////////////////////////////////////////////////////////////////

    public float[] getValue(int index)
    {
	if (!checkAllocated(index))
	{
	    throw new UnallocatedElementException();
	}

	return getValueUnchecked(index);
    }

    public void getValue(int index, float[] value)
    {
	if (!checkAllocated(index))
	{
	    throw new UnallocatedElementException();
	}

	getValueUnchecked(index, value);
    }

    public void setValue(int index, float[] value)
    {
	if (!checkAllocated(index))
	{
	    throw new UnallocatedElementException();
	}

	setValueUnchecked(index, value);
    }

    public void setValue(int index, float r, float g, float b)
    {
	if (!checkAllocated(index))
	{
	    throw new UnallocatedElementException();
	}

	setValueUnchecked(index, r, g, b);
    }

    public void clear()
    {
	super.clear();
	m_numAllocatedSegments = 0;
	m_segments = new float[STARTING_NUM_SEGMENTS][];
    }

    ////////////////////////////////////////////////////////////////////////
    // PROTECTED METHODS
    ////////////////////////////////////////////////////////////////////////

    protected int getNumAllocatedSegments()
    {
	return m_numAllocatedSegments;
    }

    protected void allocateSegment()
    {
	if (m_numAllocatedSegments == m_segments.length)
	{
	    expandSegmentsArray();
	}

	m_segments[m_numAllocatedSegments++] = new float[m_segmentSize * 3];
    }

    ////////////////////////////////////////////////////////////////////////
    // PRIVATE METHODS
    ////////////////////////////////////////////////////////////////////////

    private float[] getValueUnchecked(int index)
    {
	int segment = index >> m_segmentSizeBits;
	int offset = 3 * (index & m_segmentMask);

	float[] retval = new float[3];

	retval[0] = m_segments[segment][offset];
	retval[1] = m_segments[segment][offset + 1];
	retval[2] = m_segments[segment][offset + 2];

	return retval;
    }

    private void getValueUnchecked(int index, float[] value)
    {
	int segment = index >> m_segmentSizeBits;
	int offset = 3 * (index & m_segmentMask);

	value[0] = m_segments[segment][offset];
	value[1] = m_segments[segment][offset + 1];
	value[2] = m_segments[segment][offset + 2];
    }

    private void setValueUnchecked(int index, float[] value)
    {
	int segment = index >> m_segmentSizeBits;
	int offset = 3 * (index & m_segmentMask);

	m_segments[segment][offset] = value[0];
	m_segments[segment][offset + 1] = value[1];
	m_segments[segment][offset + 2] = value[2];
    }

    private void setValueUnchecked(int index, float r, float g, float b)
    {
	int segment = index >> m_segmentSizeBits;
	int offset = 3 * (index & m_segmentMask);

	m_segments[segment][offset] = r;
	m_segments[segment][offset + 1] = g;
	m_segments[segment][offset + 2] = b;
    }

    private void expandSegmentsArray()
    {
	float[][] oldSegments = m_segments;
	m_segments = new float[oldSegments.length * 2][];
	System.arraycopy(oldSegments, 0, m_segments, 0, oldSegments.length);
    }

    ////////////////////////////////////////////////////////////////////////
    // PRIVATE CONSTANTS
    ////////////////////////////////////////////////////////////////////////

    private static final boolean DEBUG = true;

    private static final int STARTING_NUM_SEGMENTS = 512;

    ////////////////////////////////////////////////////////////////////////
    // PRIVATE FIELDS
    ////////////////////////////////////////////////////////////////////////

    private int  m_numAllocatedSegments;
    private float[][]  m_segments = new float[STARTING_NUM_SEGMENTS][];
}
