// 
// The LibSea Graph Library.
// Copyright (C) 2000,2001,2002 The Regents of the University of California.
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
// 
// ######END_HEADER######
// 


package org.caida.libsea;

public interface GraphBuilder
{
    void     setGraphName(String name);
    void     setGraphDescription(String description);
    void     allocateNodes(int count);
    void     allocateLinks(int count);
    void     allocatePaths(int count);
    void     allocatePathLinks(int count);
    int      addLink(int source, int destination);
    int      addPath();
    void     addPathLink(int link);
    EnumerationCreator  addEnumeration(String name)
	throws DuplicateObjectException;
    int      addAttributeDefinition(String name, ValueType type,
				    int enumeration, String expr)
	throws DuplicateObjectException;
    int      addMutableAttributeDefinition(String name, ValueType type,
					   int enumeration, String expr,
					   boolean node, boolean link,
					   boolean path)
	throws DuplicateObjectException;
    AttributeCreator  addNodeAttribute(int node);
    AttributeCreator  addLinkAttribute(int link);
    AttributeCreator  addPathAttribute(int path);
    AttributeCreator  addObjectAttribute(ObjectType type, int id);
    QualifierCreator  addQualifier(String type, String name,
				   String description)
	throws DuplicateObjectException;
    int      addPresentation(String name, int display, int selector);
    DisplayCreator  addDisplay(String name);
    SelectorCreator  addSelector(String name);
    int      addFilter(String name, String expr);
    MenuCreator  addPresentationSubmenu(String label, int presentation);
    MenuCreator  addDisplaySubmenu(String label, int display);
    MenuCreator  addSelectorSubmenu(String label, int selector);
    MenuCreator  addFilterSubmenu(String label, int filter);
    MenuCreator  addAttributeSubmenu(String label, int attribute);
    Graph    endConstruction();
}
